import argparse
import logging
import os
from multiprocessing import freeze_support

import uvicorn
from core import LoggedBodyMiddleware
from environment.checks import PlatformEnvironment
from fastapi import FastAPI
from handlers.base import base_router
from handlers.meta import meta_router
from logging_config import LOGGING_CONFIG

# server startup constants
SSA_LISTEN = os.getenv("SSA_LISTEN", None)
SSA_PORT = os.getenv("SSA_PORT", None)

logging.config.dictConfig(LOGGING_CONFIG)


def get_application() -> FastAPI:
    """Make FastAPI application"""
    application = FastAPI()
    application.include_router(base_router)

    # Include router only if platform is Windows
    if PlatformEnvironment.is_windows():
        from handlers.rds import rds_router

        application.include_router(rds_router)

    # Include router only if platform is Linux
    if PlatformEnvironment.is_linux():
        from handlers.stal import stal_router

        application.include_router(stal_router)

    application.include_router(meta_router, prefix="/meta", tags=["Meta"])

    application.add_middleware(LoggedBodyMiddleware)

    return application


app = get_application()


def start_agent():
    parser = argparse.ArgumentParser(description="Run a rds_agent")
    parser.add_argument(
        "-l",
        "--listen",
        type=str,
        default="0.0.0.0",
        help="Specify the IP address on which the server listens",
    )
    parser.add_argument(
        "-p",
        "--port",
        type=int,
        default=31000,
        help="Specify the port on which the server listens",
    )
    args = parser.parse_args()

    uvicorn.run(
        "app:app",
        host=SSA_LISTEN if SSA_LISTEN else args.listen,
        port=SSA_PORT if SSA_PORT else args.port,
    )


if __name__ == "__main__":
    freeze_support()
    start_agent()
