import json
import logging
from typing import Any, Optional, AsyncGenerator, Dict

import anyio
from fastapi import Request, Response
from starlette.datastructures import MutableHeaders
from starlette.middleware.base import (
    BaseHTTPMiddleware,
    DispatchFunction,
    RequestResponseEndpoint,
)
from starlette.responses import PlainTextResponse, Response, StreamingResponse
from starlette.types import ASGIApp, Receive, Scope, Send


logger = logging.getLogger(__name__)

SECURE_FIELDS = ("password",)
SECURE_VALUE_MASK = "*" * 3


class BaseLoggingMiddleware(BaseHTTPMiddleware):
    """Base logging middleware, added log message after route info message"""

    def __init__(
        self, app: ASGIApp, dispatch: Optional[DispatchFunction] = None
    ) -> None:
        super().__init__(app, dispatch)
        self.__queue_log = []

    def add_to_logging(self, log_lvl: int, msg: str):
        """Adds a func logging to the queue"""

        def _inner():
            logger.log(log_lvl, msg)

        self.__queue_log.append(_inner)

    def safe_logging(self):
        """Print logging message from queue"""
        for func in self.__queue_log:
            func()

        self.__queue_log = []

    @staticmethod
    async def set_body(request: Request):
        """Resave body from request"""
        receive_ = await request._receive()

        async def receive():
            return receive_

        request._receive = receive

    def error_response(self, request: Request, exc: Exception) -> Response:
        """Make error response, code from ServerErrorMiddleware"""
        return PlainTextResponse("Internal Server Error", status_code=500)

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
        if scope["type"] != "http":
            await self.app(scope, receive, send)
            return

        async def call_next(request: Request) -> Response:
            app_exc: Optional[Exception] = None
            send_stream, recv_stream = anyio.create_memory_object_stream()

            async def coro() -> None:
                nonlocal app_exc

                async with send_stream:
                    try:
                        await self.app(scope, request.receive, send_stream.send)
                    except Exception as exc:
                        app_exc = exc

            task_group.start_soon(coro)

            try:
                message = await recv_stream.receive()
            except anyio.EndOfStream:
                if app_exc is not None:

                    request = Request(scope)  # from ServerErrorMiddleware
                    response = self.error_response(request, app_exc)
                    await response(scope, receive, send)

                    self.safe_logging()

                    raise app_exc
                raise RuntimeError("No response returned.")

            assert message["type"] == "http.response.start"

            async def body_stream() -> AsyncGenerator[bytes, None]:
                async with recv_stream:
                    async for message in recv_stream:
                        assert message["type"] == "http.response.body"
                        yield message.get("body", b"")

                if app_exc is not None:
                    raise app_exc

            response = StreamingResponse(
                status_code=message["status"], content=body_stream()
            )
            response.raw_headers = message["headers"]
            return response

        async with anyio.create_task_group() as task_group:
            request = Request(scope, receive=receive)
            await self.set_body(request)
            response = await self.dispatch_func(request, call_next)
            await response(scope, receive, send)
            self.safe_logging()
            task_group.cancel_scope.cancel()


class LoggedBodyMiddleware(BaseLoggingMiddleware):
    @staticmethod
    def __json_secure(body: Dict[Any, Any]) -> Dict[Any, Any]:
        """Replace value in secure key on SECURE_VALUE_MASK"""
        for field in SECURE_FIELDS:
            if field in body:
                body[field] = SECURE_VALUE_MASK
        return body

    @staticmethod
    async def get_request_body(request: Request) -> Dict[Any, Any]:
        """Return request body"""
        req_body = await request.body()
        return json.loads(req_body) if req_body else {}

    @staticmethod
    async def get_response_body(response: Response) -> Dict[Any, Any]:
        """Return response body"""
        res_body = b"".join([chunk async for chunk in response.body_iterator])
        return json.loads(res_body)

    async def dispatch(
        self, request: Request, call_next: RequestResponseEndpoint
    ) -> Response:
        json_req_body = await self.get_request_body(request)
        if json_req_body:
            self.add_to_logging(
                logging.DEBUG,
                "request: {0}".format(self.__json_secure(json_req_body)),
            )

        response = await call_next(request)

        if self.is_response_body_json(response):
            json_res_body = await self.get_response_body(response)

            if "traceback" in json_res_body:
                traceback_ = json_res_body.pop("traceback")
                self.add_to_logging(logging.ERROR, traceback_)

            self.add_to_logging(
                logging.DEBUG,
                "response: {0}".format(self.__json_secure(json_res_body)),
            )

            res_body = json.dumps(json_res_body).encode("utf-8")
            return self.make_response(response, res_body)

        return response

    @staticmethod
    def make_response(response: Response, body: bytes) -> Response:
        """Create response method"""
        headers = MutableHeaders(scope={"headers": response.headers})
        headers.raw["content-length"] = str(len(body))

        return Response(
            content=body,
            status_code=response.status_code,
            headers=headers.raw,
            media_type=response.media_type,
        )

    @staticmethod
    def is_response_body_json(response: Response) -> bool:
        """Checked response content type is json"""
        return response.headers["content-type"] == "application/json"
