import os
import platform
import subprocess


# List of VMs
OVIRT_VM = 'oVirt'
VIRTUAL_BOX_VM = ['VirtualBox', 'innotek GmbH']
HYPER_V_VM = 'Virtual Machine'
# List of supported VMs
SUPPORTED_VMS = [OVIRT_VM, *VIRTUAL_BOX_VM, HYPER_V_VM]

# Windows checks:
# Check windows hardware serial number via CMD
HARDWARE_SERIAL_NUMBER = 'WMIC BIOS GET SERIALNUMBER'
# Check windows hardware model via CMD
HARDWARE_MODEL = 'WMIC COMPUTERSYSTEM GET MODEL'
# Check windows hardware manufacturer number via CMD
HARDWARE_MANUFACTURER = 'WMIC COMPUTERSYSTEM GET MANUFACTURER'

# Linux checks:
# Checking if os is virtual machine via bash command: systemd-detect-virt
LINUX_DETECT_VIRT = 'systemd-detect-virt'
# Checking if os is virtual machine via bash command: hostnamectl
LINUX_HOSTNAMECTL = 'hostnamectl|grep -i Virtualization'
# Checking if os is virtual machine via bash command: dmesg
LINUX_DMESG = 'dmesg|grep -i hypervisor'
# Result of the check if operating system is not VM
LINUX_DETECT_VIRT_IS_PC = 'none'

# Operating systems
WINDOWS = 'Windows'
LINUX = 'Linux'

# Session states
SESSION_STATES = {"Активно": "A", "Диск": "I", "Active": "A", "Inactive": "I"}


class PlatformEnvironment:
    """Environment checks of the current operating system"""
    def __init__(self):
        self.system = platform.uname().system

    def _check_windows_platform(self, command: str) -> bool:
        """Checking if any vm from VM_LIST in output of command. Windows only."""
        try:
            result = os.popen(command).read()
            is_vm = any(vm in result for vm in SUPPORTED_VMS)
        except:
            is_vm = False
        return is_vm

    def _check_windows_serial(self) -> bool:
        """Checking if serial number from output of the command == 0:str. Windows only."""
        try:
            result = os.popen(HARDWARE_SERIAL_NUMBER).read().replace('\n', '').split()
            is_vm = True if result[1] == '0' else False
        except:
            is_vm = False
        return is_vm

    def _check_linux_platform(self, command: str) -> bool:
        """Checking if Linux system is VM from output of the bash command"""
        try:
            result = os.popen(command).read().rstrip().split(': ')[-1]
            is_vm = True if result else False
        except:
            is_vm = False
        return is_vm

    def _detect_linux_virt(self) -> bool:
        """Checking if Linux system is VM from output of the bash command"""
        try:
            result = subprocess.run(LINUX_DETECT_VIRT, stdout=subprocess.PIPE).stdout.decode().rstrip()
            is_vm = True if result != LINUX_DETECT_VIRT_IS_PC else False
        except:
            is_vm = False
        return is_vm

    def _windows_is_vm(self) -> bool:
        """Checking if Windows system is Virtual Machine"""
        is_vm = any([
            self._check_windows_serial(),
            self._check_windows_platform(HARDWARE_MANUFACTURER),
            self._check_windows_platform(HARDWARE_MODEL)
        ])
        return is_vm

    def _linux_is_vm(self) -> bool:
        """Checking if Linux system is Virtual Machine"""
        is_vm = any([
            self._detect_linux_virt(),
            self._check_linux_platform(LINUX_HOSTNAMECTL),
            self._check_linux_platform(LINUX_DMESG)
        ])
        return is_vm

    def is_vm(self) -> bool:
        """Checking if supported system is a virtual machine.
        If system is not supported raise OSError.
        """
        try:
            is_vm = eval('self._{}_is_vm()'.format(self.system.lower()))
            return is_vm

        except AttributeError:
            raise OSError('Current operating system is not supported: {}.'.format(self.system))

    def is_pc(self) -> bool:
        """Checking if the system is a physical machine.
        If system is not supported raise OSError.
        """
        return not self.is_vm()

    @staticmethod
    def is_windows() -> bool:
        """Checking if the platform is a Windows."""
        return platform.uname().system == WINDOWS

    @staticmethod
    def is_linux() -> bool:
        """Checking if the platform is a Linux."""
        return platform.uname().system == LINUX
