import traceback

from fastapi_utils.cbv import cbv
from fastapi_utils.inferring_router import InferringRouter
from models.meta import (
    MetaApps,
    MetaAvailableApps,
    MetaDeleteApp,
    MetaLogin,
    MetaLogout,
    MetaLogoutModel,
    MetaPublishApp,
    MetaPublishAppModel,
    MetaSessions,
    MetaTest,
    MetaTestConnect,
    MetaUserModel,
    MetaProgramRequestModel,
    MetaSessionRequest,
)
from services.meta import MetaService

meta_router = InferringRouter()


@cbv(meta_router)
class MetaHandler:
    """
    Class for handling requests for Meta
    """

    @meta_router.get("/test")
    def check_instance(self) -> MetaTest:
        """
        Checking the health of the meta server
        """
        service_name = MetaService().name
        response = {"service_name": service_name}

        return response

    @meta_router.get("/sessions")
    def sessions(self) -> MetaSessions:
        """
        Getting sessions list
        """
        try:
            sessions = MetaService().get_sessions()
            response = {"sessions": sessions, "error": None}
        except Exception as ex:
            response = {
                "sessions": None,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }

        return response

    @meta_router.post("/login")
    def login(self, params: MetaUserModel) -> MetaLogin:
        """
        User authorization

        - **login**: user login
        - **domain**: user domain
        - **password**: user password
        """
        try:
            is_authenticated = MetaService().login_user(params)

            response = {"auth_done": is_authenticated, "error": None}
        except Exception as ex:
            response = dict(
                auth_done=False,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return response

    @meta_router.post("/testconnect")
    def test_connect(self, params: MetaUserModel) -> MetaTestConnect:
        """
        Checking connect user

        - **login**: user login
        - **domain**: user domain
        - **password**: user password
        """
        try:
            is_connected = MetaService().test_connect(params)
            response = {"is_connected": is_connected, "error": None}
        except Exception as ex:
            response = {
                "is_connected": False,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }

        return response

    @meta_router.post("/logout")
    def logout_user(self, params: MetaLogoutModel) -> MetaLogout:
        """
        Disconnecting user

        - **session_id**: session id for logout
        """
        try:
            MetaService().logout_user(params)

            response = {"disconnect_done": True, "error": None}
        except Exception as ex:
            response = dict(
                disconnect_done=False,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return response

    @meta_router.get("/published_apps")
    def published_apps(self) -> MetaApps:
        """
        Getting published applications
        """
        try:
            apps = MetaService().get_remote_apps()

            response = {"apps": apps, "error": None}
        except Exception as ex:
            response = dict(
                apps=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return response

    # @meta_router.post("/application")
    # def publish_app(self, params: MetaPublishAppModel) -> MetaPublishApp:
    #     """
    #     Adding an application on Windows Server to the list of available

    #     - **name**: application name
    #     - **path**: path to application
    #     """
    #     try:
    #         MetaService().publish_app(params)

    #         response = {"is_created": True, "error": None}
    #     except Exception as ex:
    #         response = dict(
    #             is_created=None,
    #             error=str(ex),
    #             traceback=traceback.format_exc(),
    #         )

    #     return response

    # @meta_router.delete("/application/{name}")
    # def delete_app(self, name: str) -> MetaDeleteApp:
    #     """
    #     Removing an application on Windows Server from the list of available

    #     - **name**: application name
    #     """
    #     try:
    #         MetaService().delete_app(name)

    #         response = {"is_deleted": True, "error": None}
    #     except Exception as ex:
    #         response = dict(
    #             is_deleted=None,
    #             error=str(ex),
    #             traceback=traceback.format_exc(),
    #         )

    #     return response

    @meta_router.get("/available_apps")
    def check_available_apps(self) -> MetaAvailableApps:
        """
        Getting publishable Windows Server applications
        """
        try:
            app_list = MetaService().get_list_startapps()
            response = {"available_apps": app_list, "error": None}
        except Exception as ex:
            response = dict(
                available_apps=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return response

    @meta_router.post("/sessionrequest")
    def session_request(self, user: MetaUserModel) -> MetaSessionRequest:
        """
        Method for creating user session in STAL
        """
        try:
            # session_id может быть -1, если сессия не была создана
            session_id = MetaService().session_request(user)  # dbus.Int32

            result = {
                "session_id": session_id,
                "error": "STAL session not created!" if session_id == -1 else None,
            }
        except Exception as ex:
            result = {
                "session_id": -1,
                "error": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result

    @meta_router.post("/programrequest")
    def program_request(self, data: MetaProgramRequestModel) -> MetaSessionRequest:
        """
        Method for creating user session in STAL in single program mode
        """
        try:
            session_id = MetaService().program_request(data)  # dbus.Int32

            result = {
                "session_id": session_id,
                "error": "STAL session not created!" if session_id == -1 else None,
            }
        except Exception as ex:
            result = {
                "session_id": -1,
                "error": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result
