from fastapi_utils.cbv import cbv
from fastapi_utils.inferring_router import InferringRouter

from models.rds import (
    RdsUserModel,
    RdsLogoutModel,
    RdsRemoteAppModel,
    RemoteAppUsersModel,
)
from utils.rds import check_instance_is_rds
from services.rds import RdsService


rds_router = InferringRouter()


@cbv(rds_router)
class RdsHandler:
    """
    Class for handling requests for MS RDS
    """

    @rds_router.get("/rds/test")
    def check_rds_instance(self):
        """method for handling server rds check request.

        Returns:
            response (dict): json response
        """

        response = check_instance_is_rds()

        return response

    @rds_router.get("/rds/sessions")
    def rds_sessions(self):
        """method for handling a request to get rds user sessions.

        Returns:
            response (dict): json response
        """

        response = RdsService().get_sessions()

        return response

    @rds_router.post("/rds/login")
    def rds_login(self, params: RdsUserModel):
        """method to handle connection request to rds.

        Args:
            params (RdsUserModel): auth params

        Returns:
            response (dict): json response
        """

        response = RdsService().login(params)

        return response

    @rds_router.post("/rds/testconnect")
    def rds_test_connect(self, params: RdsUserModel):
        """method for handling request for a test connection to RDS.

        Args:
            params (RdsUserModel): auth params

        Returns:
            response (dict): json response
        """

        response = RdsService().connect(params)

        return response

    @rds_router.post("/rds/disconnect")
    def rds_logout_user(self, params: RdsLogoutModel):
        """method for handling a request to disconnect a user.

        Args:
            params (RdsLogoutModel): auth params

        Returns:
            response (dict): json response
        """

        response = RdsService().logout_user(params)

        return response

    @rds_router.get("/rds/collections")
    def rds_collections(self):
        """method for handling a request to get rds collections.

        Returns:
            response (dict): json response
        """

        response = RdsService().get_collections()

        return response

    @rds_router.post("/rds/apps")
    def rds_apps(self, params: RdsRemoteAppModel):
        """method for handling a request to get published remote apps.

        Args:
            params (RdsRemoteAppModel): contains collection name

        Returns:
            response (dict): json response
        """

        response = RdsService().get_remote_apps(params)

        return response

    @rds_router.post("/rds/apps/users")
    def rds_app_users(self, params: RemoteAppUsersModel):
        """method for handling a request to getting users that have access
         to published app by collection name and remote app alias.

        Args:
            params (RemoteAppUsersModel): contains collection name
                                        and remote app alias

        Returns:
            response (dict): json response
        """

        response = RdsService().get_remote_app_users(params)

        return response

    @rds_router.get("/rds/collection_info/{collection_name}")
    def collection_info(self, collection_name: str):
        """method for handling a request to get collection info by collection name

        Returns:
            response (dict): json response
        """

        response = RdsService().get_collection_info(collection_name)

        return response
