import os
import socket
from typing import Any, Dict, List

import pythoncom
import win32com.client
import win32security
import win32ts
from win32api import CloseHandle

from models.meta import (
    MetaLogoutModel,
    MetaPublishAppModel,
    MetaUserModel,
    MetaProgramRequestModel,
)
from utils.meta.base import extract_ip
from utils.meta.rds import ApplicationRegEdit


# Local server handler
WTS_CURRENT_SERVER_HANDLE = 0
# Initial programm for RDP sessions
RDPINIT_EXE = "rdpinit.exe"
# State active message
STATE_ACTIVE = "STATE_ACTIVE"
# State connected message
STATE_CONNECTED = "STATE_CONNECTED"
# Win SessionId association
SESSION_ID_STATES = {
    -1: "STATE_INVALID",
    0: STATE_ACTIVE,
    1: STATE_CONNECTED,
    2: "STATE_CONNECTQUERY",
    3: "STATE_SHADOW",
    4: "STATE_DISCONNECTED",
    5: "STATE_IDLE",
    6: "STATE_LISTEN",
    7: "STATE_RESET",
    8: "STATE_DOWN",
    9: "STATE_INIT",
}
# Session states for termidesk Inactive/Activeresponse
SESSION_STATES = "IA"


class MetaService:
    """
    Сервисный слой для взаимодействия с RDS
    """

    __name = "rds"

    @property
    def name(self) -> str:
        return self.__name

    def __get_current_state(self, state: int) -> str:
        """Getting session state using a numeric session value"""
        return SESSION_STATES[
            SESSION_ID_STATES.get(state)
            # Сreated session has SessionState is STATE_ACTIVE,
            # if reconected to session has SessionState is STATE_CONNECTED
            in (STATE_ACTIVE, STATE_CONNECTED)
        ]

    def __get_session_info(self, session_id: int, info_class: int) -> Any:
        """Getting info about the session using the info class for the session id"""
        return win32ts.WTSQuerySessionInformation(
            WTS_CURRENT_SERVER_HANDLE, session_id, info_class
        )

    def get_sessions(self) -> List[Dict[str, Any]]:
        """Getting RDS sessions list."""
        sessions = []
        host_server = extract_ip()
        for session in win32ts.WTSEnumerateSessions():
            session_id = session["SessionId"]
            username = self.__get_session_info(session_id, win32ts.WTSUserName)
            if not username:  # sessions not bound to a user have no username
                continue
            # if the session state is inactive initial program is always empty
            initial_program = self.__get_session_info(
                session_id, win32ts.WTSInitialProgram
            )
            sessions.append(
                {
                    "username": username,
                    "session_id": session_id,
                    "host_server": host_server,  # for logoff in future
                    "state": self.__get_current_state(session["State"]),
                    "is_desktop": initial_program != RDPINIT_EXE,
                }
            )

        return sessions

    def login_user(self, params: MetaUserModel) -> bool:
        """Authorization method on RDS

        Args:
            params (MetaUserModel): auth params
        """
        # LogonUser() return a handle to a token that represents the specified user.
        # Returned handle is a primary token that you can use in calls to the
        # CreateProcessAsUser() function.
        # If you specify the LOGON32_LOGON_NETWORK flag, LogonUser() returns
        # an impersonation token that you cannot use in CreateProcessAsUser()
        # unless you call DuplicateTokenEx() to convert it to a primary token.
        # LOGON32_PROVIDER_WINNT40: Use the NTLM logon provider.
        token = win32security.LogonUser(
            params.login,
            params.domain,
            params.password,
            win32security.LOGON32_LOGON_NETWORK,
            win32security.LOGON32_PROVIDER_WINNT40,
        )
        is_authenticated = bool(token)
        CloseHandle(token)

        return is_authenticated

    def test_connect(self, params: MetaUserModel) -> bool:
        """Check connect user on RDS

        Args:
            params (MetaUserModel): connect params
        """
        # TODO Пока не понятно как правильно делать проверку тестового подключения по RDP.
        # Оставим текущую логику закомментированной.

        # RDS_HOSTNAME = '192.168.3.13'
        # login = params.login
        # pwd = params.password

        # os.system(
        #     'cmd/c "cmdkey /generic:"{hostname}" /user:"{login}" \
        #         /pass:"{pwd}""'.format(hostname=RDS_HOSTNAME, login=login, pwd=pwd)
        #     )
        # os.system('cmd/c "mstsc /v:{hostname}"'.format(hostname=RDS_HOSTNAME))
        # os.system('cmd/c "cmdkey /delete:{hostname}"'.format(hostname=RDS_HOSTNAME))

        # Временное решение делать проверку посредством попытки авторизации на AD.
        is_connected = self.login_user(params)

        return is_connected

    def logout_user(self, params: MetaLogoutModel) -> None:
        """Disconnect user on RDS

        Args:
            params (MetaLogoutModel): logout params
        """
        session_id = params.session_id
        # In future use host server
        # Example: logoff /VM <session id> /server:<host server>
        os.system('cmd/c "echo y|LOGOFF /VM {sessin_id}"'.format(sessin_id=session_id))

    def get_remote_apps(self) -> List[str]:
        """Getting published remote RDS apps"""
        apps = ApplicationRegEdit().get_apps()

        return apps

    def publish_app(self, params: MetaPublishAppModel) -> None:
        """Application Publishing method on RDS

        Args:
            params (MetaPublishAppModel): publish params
        """
        ApplicationRegEdit(params.name).create_app(params.path)

    def delete_app(self, name: str) -> None:
        """Deleting an application to RDS

        Args:
            name (str): application name
        """
        ApplicationRegEdit(name).delete_app()

    def get_list_startapps(self) -> Dict[str, str]:
        """Getting list of apps from Start Menu. Available apps for publish"""
        pythoncom.CoInitialize()
        # We need to do a winsrv version check. The path in each version of winsrv may be different.
        start_menu_path = os.path.join(
            os.environ["PROGRAMDATA"], "Microsoft", "Windows", "Start Menu", "Programs"
        )
        app_list = {}
        shell = win32com.client.Dispatch("WScript.Shell")

        for root, dirs, files in os.walk(start_menu_path):
            for file in files:
                if file.endswith(".lnk"):
                    path = os.path.join(root, file)
                    name = os.path.splitext(os.path.basename(file))[0]
                    shortcut = shell.CreateShortCut(path)
                    target_path = shortcut.TargetPath
                    app_list[name] = target_path

        return app_list

    def session_request(self, params: MetaUserModel):  # dbus.Int32
        """
        Method for creating user session in RDS
        """
        raise NotImplementedError("The method is not intended to be called on this OS.")

    def program_request(self, params: MetaProgramRequestModel):  # dbus.Int32:
        """
        Method for creating user session in RDS in single program mode
        """
        raise NotImplementedError("The method is not intended to be called on this OS.")
