import socket
from typing import Any, Dict, List

import dbus
import json

from models.meta import (
    MetaUserModel,
    MetaLogoutModel,
    MetaProgramRequestModel,
    MetaPublishAppModel,
)
from utils.meta.base import extract_ip

# Глубина цвета
DEPTH = 24
# Разрешение экрана
DISPLAY_WIDTH, DISPLAY_HEIGHT = 1024, 768

# Session states for termidesk Inactive/Activeresponse
SESSION_STATES = "IA"


class MetaService:
    """
    Сервисный слой для взаимодействия со STAL
    """

    __bus = None
    __stal = None
    __stal_iface = None
    __session_info: dict = {}
    __name = "stal"

    # Class implemented via Singleton
    def __new__(cls):
        if not hasattr(cls, "_instance"):
            cls._instance = super(MetaService, cls).__new__(cls)
        return cls._instance

    def __check_stal_connect(self) -> None:
        """
        Method for check STAL connect success
        """
        if self.__stal_iface.stalStatus() != True:
            raise ValueError("No connecting to STAL")

    @property
    def stal_iface(self) -> dbus.Interface:
        if not self.__stal_iface:
            self.__bus = dbus.SystemBus()
            self.__stal = self.__bus.get_object("ru.uveon.stal", "/ru/uveon/stal")
            self.__stal_iface = dbus.Interface(self.__stal, "ru.uveon.stal")
            return self.__stal_iface

        self.__check_stal_connect()
        return self.__stal_iface

    @property
    def name(self) -> str:
        return self.__name

    def get_session_info(self, session_id: int) -> Dict[str, Any]:
        """
        Gets session information
        """
        session_info: dbus.Boolean = self.stal_iface.stalSessionInfoJson(session_id)
        return session_info

    def __load_session_info(self, session_id) -> None:
        """
        Records session information
        """
        info: str = self.get_session_info(session_id)
        self.__session_info = json.loads(info)

    def __clear_session_info(self) -> None:
        """
        Removes session information
        """
        self.__session_info = None

    def __get_session_state(self) -> str:
        """
        Get session state
        """
        is_session_active = self.__session_info["pid3"] != 0
        return SESSION_STATES[is_session_active]

    def __get_session_is_app(self) -> bool:
        """
        Get the value of whether the session is an application
        Parameter value 'seamless:mode':
        if mode == false - terminal
        if mode == true - app
        """
        is_app_mode = self.__session_info["seamless:mode"]
        if isinstance(is_app_mode, str):  # special for STAL V0.3
            is_app_mode = is_app_mode == "true"
        return is_app_mode

    def get_sessions(self) -> List[Dict[str, Any]]:
        """
        Method for gets STAL sessions
        """
        # Example:
        # dbus.Array(
        #     [
        #         dbus.Struct(
        #             (dbus.String("test1"), dbus.UInt32(59101)), signature=None
        #         )
        #     ],
        #     signature=dbus.Signature("(su)"),
        # )
        dbus_sessions: dbus.Array[dbus.Struct] = self.stal_iface.stalSessions()
        host_server = extract_ip()
        sessions = []
        for session in dbus_sessions:
            username = session[0]
            session_id = session[1]

            self.__load_session_info(session_id)
            state = self.__get_session_state()
            is_desktop = not self.__get_session_is_app()
            self.__clear_session_info()

            sessions.append(
                {
                    "username": username,
                    "session_id": session_id,
                    "host_server": host_server,  # for logoff in future
                    "state": state,
                    "is_desktop": is_desktop,
                }
            )

        return sessions

    def test_connect(self, params: MetaUserModel) -> dbus.Boolean:
        """
        Method for authentication
        """
        is_success: dbus.Boolean = self.stal_iface.stalTestConnect(
            params.login,  # user
            params.domain,  # domain
            params.password,  # pass
        )
        return is_success

    def login_user(self, params: MetaUserModel) -> dbus.Boolean:
        """
        Method for start of a new user session
        """
        is_success: dbus.Boolean = self.stal_iface.stalLogin(
            params.login,  # user
            params.domain,  # domain
            params.password,  # pass
        )
        return is_success

    def logout_user(self, params: MetaLogoutModel) -> dbus.Boolean:
        """
        Method for shutdown user session
        Session closes assync ~600ms
        """
        is_success: dbus.Boolean = self.stal_iface.stalSessionShutdown(
            params.session_id,
        )
        return is_success

    def session_request(self, params: MetaUserModel) -> dbus.Int32:
        """
        Method for creating user session in STAL
        """
        # session_id может быть -1, если сессия не была создана
        session_id: dbus.Int32 = self.stal_iface.busSessionRequest(
            params.login,  # user
            params.password,  # pass
            DEPTH,  # depth
            DISPLAY_WIDTH,  # width
            DISPLAY_HEIGHT,  # height
        )
        return session_id

    def program_request(self, params: MetaProgramRequestModel) -> dbus.Int32:
        """
        Method for creating user session in STAL in single program mode
        """
        session_id: dbus.Int32 = self.stal_iface.busProgramRequest(
            params.login,  # user
            params.domain,  # domain
            params.password,  # pass
            DISPLAY_WIDTH,  # width
            DISPLAY_HEIGHT,  # height
            params.program,  # program
        )
        return session_id

    def get_remote_apps(self) -> dbus.Array:
        """Getting published remote apps from STAL.

        Returns:
            result (dict): json response
        """
        remote_apps: dbus.Array = self.stal_iface.stalProgramList()
        return remote_apps

    def publish_app(self, params: MetaPublishAppModel) -> None:
        """Application Publishing method on STAL

        Args:
            params (MetaPublishAppModel): publish params
        """
        raise NotImplementedError("The method is not intended to be called on this OS.")

    def delete_app(self, name: str) -> None:
        """Deleting an application to STAL

        Args:
            name (str): application name
        """
        raise NotImplementedError("The method is not intended to be called on this OS.")

    def get_list_startapps(self) -> Dict[str, str]:
        """Getting list of apps from Start Menu. Available apps for publish"""
        raise NotImplementedError("The method is not intended to be called on this OS.")
