import os
import traceback

import win32security
from win32api import CloseHandle

from utils.rds import (
    psh_request_handler,
    WMIRemoteAppHandler,
)
from models.rds import (
    RdsUserModel,
    RdsLogoutModel,
    RdsRemoteAppModel,
    RemoteAppUsersModel,
)

GET_REMOTE_APP_USERS_CMD = (
    'Get-RDRemoteApp -CollectionName "{}" -Alias "{}" | Format-List UserGroups'
)


class RdsService:
    def get_sessions(self) -> dict:
        """function returning active rds sessions.

        Returns:
            result (dict): json response
        """
        try:
            sessions = WMIRemoteAppHandler().get_sessions()
            result = {"sessions": sessions, "error": None}

        except Exception as ex:
            result = {
                "sessions": None,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }

        return result

    def login(self, params: RdsUserModel) -> dict:
        """authorization function in rds

        Args:
            params (RdsInputModel): auth params

        Returns:
            result (dict): json response
        """

        try:
            # LogonUser() return a handle to a token that represents the specified user.
            # Returned handle is a primary token that you can use in calls to the
            # CreateProcessAsUser() function.
            # If you specify the LOGON32_LOGON_NETWORK flag, LogonUser() returns
            # an impersonation token that you cannot use in CreateProcessAsUser()
            # unless you call DuplicateTokenEx() to convert it to a primary token.
            # LOGON32_PROVIDER_WINNT40: Use the NTLM logon provider.
            token = win32security.LogonUser(
                params.login,
                params.domain,
                params.password,
                win32security.LOGON32_LOGON_NETWORK,
                win32security.LOGON32_PROVIDER_WINNT40,
            )
            authenticated = bool(token)
            CloseHandle(token)

            result = dict(auth_done=authenticated, error=None)
        except Exception as ex:
            result = dict(
                auth_done=False,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result

    def connect(self, params: RdsUserModel) -> dict:
        """method that return response of trying to connect to rds

        Args:
            params (RdsUserModel): connect params

        Returns:
            result (dict): json response
        """
        try:

            # TODO Пока не понятно как правильно делать проверку тестового подключения по RDP.
            # Оставим текущую логику закомментированной.

            # RDS_HOSTNAME = '192.168.3.13'
            # login = params.login
            # pwd = params.password

            # os.system(
            #     'cmd/c "cmdkey /generic:"{hostname}" /user:"{login}" \
            #         /pass:"{pwd}""'.format(hostname=RDS_HOSTNAME, login=login, pwd=pwd)
            #     )
            # os.system('cmd/c "mstsc /v:{hostname}"'.format(hostname=RDS_HOSTNAME))
            # os.system('cmd/c "cmdkey /delete:{hostname}"'.format(hostname=RDS_HOSTNAME))

            # Временное решение делать проверку посредством попытки авторизации на AD.
            rds_login_response = self.login(params)
            result = {"is_connected": True, "error": None}
            if not rds_login_response["auth_done"]:
                result = {
                    "is_connected": False,
                    "error": str(rds_login_response["error"]),
                }
        except Exception as ex:
            result = {
                "is_connected": False,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }

        return result

    def logout_user(self, params: RdsLogoutModel) -> dict:
        """method disconnect user from RDS

        Args:
            params (RdsInputModel): logout params

        Returns:
            result (dict): json response
        """
        try:
            session_id = params.session_id
            # In future use host server
            # Example: logoff /VM <session id> /server:<host server>
            os.system(
                'cmd/c "echo y|LOGOFF /VM {sessin_id}"'.format(sessin_id=session_id)
            )

            result = dict(disconnect_done=True, error=None)
        except Exception as ex:
            result = dict(
                disconnect_done=False,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result

    def get_collections(self) -> dict:
        """Getting RDS collections.

        Returns:
            result (dict): json response
        """
        try:
            collections = WMIRemoteAppHandler().get_collections()
            result = dict(collections=collections, error=None)

        except Exception as ex:
            result = dict(
                collections=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result

    def get_remote_app_users(self, params: RemoteAppUsersModel) -> dict:
        """Getting users that have access to published
         app by collection name and remote app alias.

        Args:
            params (RemoteAppUsersModel): contains collection name
                                        and remote app alias

        Returns:
            result (dict): json response
        """
        try:
            collection = params.collection
            app_alias = params.app_alias
            # Getting users list using powershell handler. Todo: soon to be replaced with WMI handler.
            output = psh_request_handler(
                GET_REMOTE_APP_USERS_CMD.format(collection, app_alias)
            )
            app_users = [users.lstrip("UserGroups : ") for users in output]
            result = dict(app_users=app_users, error=None)

        except Exception as ex:
            result = dict(
                app_users=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result

    def get_remote_apps(self, params: RdsRemoteAppModel) -> dict:
        """Getting published remote apps from RDS by collection name.

        Args:
            params (RdsRemoteAppModel): contains collection name

        Returns:
            result (dict): json response
        """
        try:
            collection = params.collection
            output = WMIRemoteAppHandler().get_remote_apps(collection)
            result = dict(apps=output, error=None)

        except Exception as ex:
            result = dict(
                apps=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result

    def get_collection_info(self, collection_name: str):
        """Getting collection info by collection name

        Args:
            collection_name (str): contains collection name

        Returns:
            result (dict): json response
        """
        try:
            collection_info = WMIRemoteAppHandler().get_collection_info(collection_name)

            result = dict(collection_info=collection_info, error=None)
        except Exception as ex:
            result = dict(
                collection_info=None,
                error=str(ex),
                traceback=traceback.format_exc(),
            )

        return result
