import traceback
from typing import Any, Dict, List

import dbus

# FIXME: данная реализация предназначена для корректного выполнения тестов
try:
    from models.stal import StalLogoutModel, StalUserModel, StalRemoteAppModel
except ImportError:
    from session_agent.models.stal import StalLogoutModel, StalUserModel


# Глубина цвета
DEPTH = 24
# Разрешение экрана
DISPLAY_WIDTH, DISPLAY_HEIGHT = 1024, 768
# Абривиатура состояния активной сессии
SESSION_ACTIVE_STATE = "A"


class StalService:
    """
    Сервис взаимодействия со STAL
    """

    __bus = None
    __stal = None
    __stal_iface = None

    # Class implemented via Singleton
    def __new__(cls):
        if not hasattr(cls, "_instance"):
            cls._instance = super(StalService, cls).__new__(cls)
        return cls._instance

    @property
    def stal_iface(self):
        if not self.__stal_iface:
            self.__bus = dbus.SystemBus()
            self.__stal = self.__bus.get_object("ru.uveon.stal", "/ru/uveon/stal")
            self.__stal_iface = dbus.Interface(self.__stal, "ru.uveon.stal")
            return self.__stal_iface

        self.__check_stal_connect()
        return self.__stal_iface

    def status(self) -> Dict[str, str]:
        """
        Method for gets STAL status
        """
        try:
            _ = self.stal_iface
            result = {
                "is_stal": True,
                "error": None,
            }
        except Exception as ex:
            result = {
                "is_stal": None,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result

    def version(self) -> Dict[str, str]:
        """
        Method for gets STAL versions
        """
        try:
            # Example: "STAL v.${MAJOR}.${MINOR}"
            version: dbus.String = self.stal_iface.stalTest()
            result = {"version": version, "message": "OK!"}
        except Exception as ex:
            result = {
                "version": None,
                "message": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result

    def __get_session_info(self, *, id: int, username: str, state: str) -> dict:
        """
        Create session info dict element
        """
        return {
            "username": username,
            "session_id": id,
            "state": state,
        }

    def sessions(self) -> Dict[str, Any]:
        """
        Method for gets STAL sessions
        """
        try:
            # Example:
            # dbus.Array(
            #     [
            #         dbus.Struct(
            #             (dbus.String("test1"), dbus.UInt32(59101)), signature=None
            #         )
            #     ],
            #     signature=dbus.Signature("(su)"),
            # )
            sessions: dbus.Array[dbus.Struct] = self.stal_iface.stalSessions()
            sessions_attributes: List[Dict[str, Any]] = [
                self.__get_session_info(
                    id=item[1],
                    username=item[0],
                    state=SESSION_ACTIVE_STATE,
                )
                for item in sessions
            ]
            result = {"sessions": sessions_attributes, "error": None}
        except Exception as ex:
            result = {
                "sessions": None,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result

    def test_connect(self, params: StalUserModel) -> Dict[str, str]:
        """
        Method for authentication
        """
        try:
            is_success: dbus.Boolean = self.stal_iface.stalTestConnect(
                params.login,  # user
                params.domain,  # domain
                params.password,  # pass
            )
            result = {"is_connected": is_success, "error": None}
        except Exception as ex:
            result = {
                "is_connected": False,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result

    def login_user(self, params: StalUserModel) -> Dict[str, str]:
        """
        Method for start of a new user session
        """
        try:
            is_success: dbus.Boolean = self.stal_iface.stalLogin(
                params.login,  # user
                params.domain,  # domain
                params.password,  # pass
            )
            result = {"auth_done": is_success, "error": None}
        except Exception as ex:
            result = {
                "auth_done": False,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result

    def disconnect_user(self, params: StalLogoutModel) -> Dict[str, str]:
        """
        Method for disconnect user
        """
        try:
            uint32_session_id = dbus.UInt32(params.session_id)
            is_success: dbus.Boolean = self.stal_iface.stalDisconnect(
                uint32_session_id,
            )
            result = {"message": "OK!" if is_success else "BAD!"}
        except Exception as ex:
            result = {
                "message": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result

    def shutdown_user(self, params: StalLogoutModel) -> Dict[str, str]:
        """
        Method for shutdown user session
        Session closes assync ~600ms
        """
        try:
            is_success: dbus.Boolean = self.stal_iface.stalSessionShutdown(
                params.session_id,
            )
            result = {"disconnect_done": is_success, "error": None}
        except Exception as ex:
            result = {
                "disconnect_done": False,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result

    def session_request(self, params: StalUserModel) -> Dict[str, str]:
        """
        Method for creating user session in STAL
        """
        try:
            # session_id может быть -1, если сессия не была создана
            session_id: dbus.Int32 = self.stal_iface.busSessionRequest(
                params.login,  # user
                params.password,  # pass
                DEPTH,  # depth
                DISPLAY_WIDTH,  # width
                DISPLAY_HEIGHT,  # height
            )

            result = {
                "session_id": session_id,
                "error": "STAL session not created!" if session_id == -1 else None,
            }
        except Exception as ex:
            result = {
                "session_id": -1,
                "error": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result

    def program_request(self, params: StalRemoteAppModel) -> Dict[str, str]:
        """
        Method for creating user session in STAL in single program mode
        """
        try:
            session_id: dbus.Int32 = self.stal_iface.busProgramRequest(
                params.login,  # user
                params.domain,  # domain
                params.password,  # pass
                DISPLAY_WIDTH,  # width
                DISPLAY_HEIGHT,  # height
                params.program,  # program
            )

            result = {
                "session_id": session_id,
                "error": "STAL session not created!" if session_id == -1 else None,
            }
        except Exception as ex:
            result = {
                "session_id": False,
                "error": f"Error: {ex}",
                "traceback": traceback.format_exc(),
            }
        return result

    def __check_stal_connect(self) -> None:
        """
        Method for check STAL connect success
        """
        if self.__stal_iface.stalStatus() != True:
            raise ValueError("No connecting to STAL")

    def get_remote_apps(self) -> dict:
        """Getting published remote apps from STAL.

        Returns:
            result (dict): json response
        """
        try:
            remote_apps: dbus.Array = self.stal_iface.stalProgramList()
            result = {"apps": remote_apps, "error": None}
        except Exception as ex:
            result = {
                "apps": None,
                "error": str(ex),
                "traceback": traceback.format_exc(),
            }
        return result
