import winreg as wrg
from contextlib import suppress
from itertools import count
from typing import List

# Zero is always passed to the API winreg
RESERVED = 0
# Registry value is disabled
REG_DISABLED = 0
# Registry value is enabled
REG_ENABLED = 1
# реестр локальной машины
REG_LOCATION = wrg.HKEY_LOCAL_MACHINE
# путь в реестре до списка приложений
APP_REG_PATH = "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion\\Terminal Server\\TSAppAllowList\\Applications"


class ApplicationRegEdit:
    """Сlass for application interaction with the registry"""

    def __init__(self, app_name: str = "") -> None:
        self._app_name: str = app_name

    def __create_key(self, name: str, type_, value) -> None:
        """Creates a key in the registry"""
        root_path = APP_REG_PATH + "\\" + self._app_name

        with wrg.CreateKey(REG_LOCATION, root_path) as reg:
            wrg.SetValueEx(reg, name, RESERVED, type_, value)

    def _create_dword_key(self, name: str, value) -> None:
        """Creates a REG_DWORD key in the registry"""
        self.__create_key(name, wrg.REG_DWORD, value)

    def _create_sz_key(self, name: str, value) -> None:
        """Creates a REG_SZ key in the registry"""
        self.__create_key(name, wrg.REG_SZ, value)

    def __is_path_exists(self) -> bool:
        """Checks is a path exists"""
        root_path = APP_REG_PATH + "\\" + self._app_name
        try:
            wrg.OpenKey(REG_LOCATION, root_path)
            return True
        except FileNotFoundError:
            return False

    def create_app(self, app_path: str) -> None:
        """Creates an entry with application information in the registry"""
        if self.__is_path_exists():
            raise Exception("An application with the same name already exists.")

        try:
            self._create_sz_key("Name", self._app_name)
            self._create_sz_key("Path", app_path)
            self._create_sz_key("VPath", app_path)
            self._create_sz_key("RequiredCommandLine", "")
            self._create_dword_key("CommandLineSetting", REG_ENABLED)
            self._create_sz_key("IconPath", app_path)
            self._create_dword_key("IconIndex", REG_DISABLED)
            self._create_dword_key("ShowInTSWA", REG_DISABLED)
        except:
            with suppress(OSError):
                self.delete_app()
            raise

    def update_app(self, app_path: str) -> None:
        """Updates an application in the registry"""
        if not self.__is_path_exists():
            raise Exception("An application with that name does not exist.")

        self._create_sz_key("Path", app_path)
        self._create_sz_key("VPath", app_path)
        self._create_sz_key("IconPath", app_path)

    def delete_app(self) -> None:
        """Removes an application from the registry"""
        with wrg.OpenKey(REG_LOCATION, APP_REG_PATH) as reg:
            wrg.DeleteKey(reg, self._app_name)

    @staticmethod
    def get_apps() -> List[str]:
        """
        Getting published remote apps using regedit

        Returns: list of published remote apps by their collection name.
        """
        remote_app = []
        with suppress(OSError), wrg.OpenKey(REG_LOCATION, APP_REG_PATH) as access_key:
            remote_app.extend((wrg.EnumKey(access_key, i) for i in count()))

        return remote_app
