import subprocess

import pythoncom
import wmi

from environment.checks import PlatformEnvironment

# List with proc names for checking rds instance
RDS_PROC_NAMES = ["tssdis.exe"]
# Remote application process alias
REMOTE_APP_PROC_NAME = "rdpinit.exe"
# Message if platform not Windows
NOT_WIN_MESSAGE = "Platform is not Windows!"
# State active message
STATE_ACTIVE = "STATE_ACTIVE"
# State connected message
STATE_CONNECTED = "STATE_CONNECTED"
# Win SessionId association
SESSION_ID_STATES = {
    -1: "STATE_INVALID",
    0: STATE_ACTIVE,
    1: STATE_CONNECTED,
    2: "STATE_CONNECTQUERY",
    3: "STATE_SHADOW",
    4: "STATE_DISCONNECTED",
    5: "STATE_IDLE",
    6: "STATE_LISTEN",
    7: "STATE_RESET",
    8: "STATE_DOWN",
    9: "STATE_INIT",
}
# Session states for termidesk Inactive/Active
SESSION_STATES = "IA"


def check_instance_is_rds() -> dict:
    """function returns the answer whether the server is rds.

    Returns:
        response (dict): json response
    """

    result = True
    # Making dict "check_list" through dict comprehension to further
    # check the existence of rds processes
    check_list = {proc_name: False for proc_name in RDS_PROC_NAMES}

    try:
        import psutil

        for process in psutil.process_iter():
            if process.name() in RDS_PROC_NAMES:
                check_list.update({process.name(): True})
        if not all(check_list.values()):
            result = False
        response = dict(is_rds=result, error=None)
    except Exception as ex:
        response = dict(
            is_rds=False,
            error=str(ex) if PlatformEnvironment.is_windows() else NOT_WIN_MESSAGE,
        )

    return response


def win_cli_output_decoder(win_string: str) -> str:
    """function returns a string with normal Cyrillic characters.
    Functionality will be updated.

    Args:
        win_string (string): cp1251 encoded string

    Returns:
        response (string): cp886 encoded string
    """

    return win_string.encode("cp1251").decode("cp866")


def psh_request_handler(command: str) -> list:
    """Powershell output handler. Method will be replaced with WMI handler later"""
    output = (
        subprocess.run(
            ["powershell", "-Command", command],
            universal_newlines=True,
            stdout=subprocess.PIPE,
        )
        .stdout.strip()
        .splitlines()
    )

    return output


class WMIRemoteAppHandler:
    """This class represent a methods for retrieving
    data from published applications using WMI.
    """

    _remoteapp_namespace = "root\\cimv2\\rdms"
    _termserv_namespace = "root\\cimv2\\TerminalServices"
    _cimv2_namespace = "root\\cimv2"
    _wmi_handler = None
    _wmi_termserv_handler = None
    _wmi_cimv2_handler = None

    def __init__(self):
        if not self._wmi_handler:
            pythoncom.CoInitialize()
            self._wmi_handler = wmi.WMI(namespace=self._remoteapp_namespace)
            self._wmi_termserv_handler = wmi.WMI(namespace=self._termserv_namespace)
            self._wmi_cimv2_handler = wmi.WMI(namespace=self._cimv2_namespace)

    def __get_collections_alias_by_remote_desktop(self) -> tuple:
        collections = self._wmi_termserv_handler.Win32_RDCentralPublishedRemoteDesktop()
        return tuple(
            collection.Alias for collection in collections if collection.ShowInPortal
        )

    def get_collections(self) -> dict:
        """
        Getting collections.

        Returns: dict of RDS collections aliases.
        """
        wmi_collections = (
            self._wmi_termserv_handler.Win32_RDCentralPublishedRemoteDesktop()
        )
        # Alias - Alias of the desktop.
        # ShowInPortal - True if this application should be shown in the TS Web Access; otherwise, False.
        collections = [
            {
                "name": collection.Alias,
                "is_desktop": collection.ShowInPortal,
            }
            for collection in wmi_collections
        ]

        return collections

    def get_sessions(self) -> dict:
        desktop_collections = self.__get_collections_alias_by_remote_desktop()
        wmi_sessions = self._wmi_cimv2_handler.Win32_SessionDirectorySessionEx()

        sessions = []
        for session in wmi_sessions:
            is_desktop = session.CollectionAlias in desktop_collections
            is_application = (
                session.CollectionAlias not in desktop_collections
                and session.ApplicationType == REMOTE_APP_PROC_NAME
            )
            if is_desktop or is_application:
                sessions.append(
                    {
                        "username": session.UserName,
                        "session_id": session.SessionID,
                        "host_server": session.HostServer,  # for logoff in future
                        "state": SESSION_STATES[
                            SESSION_ID_STATES.get(session.SessionState)
                            # Сreated session has SessionState is STATE_ACTIVE,
                            # if reconected to session has SessionState is STATE_CONNECTED
                            in (STATE_ACTIVE, STATE_CONNECTED)
                        ],
                        "is_desktop": is_desktop,
                    }
                )

        return sessions

    def get_remote_apps(self, collection: str) -> list:
        """Getting published remote apps using win32 class Win32_RDMSPublishedApplication.

        Args:
            - collection (str): RDS collection alias.

        Returns: list of published remote apps by their collection name.

        """
        remote_apps = self._wmi_handler.Win32_RDMSPublishedApplication(
            PoolName=collection
        )
        remote_apps_info = []
        for app in remote_apps:
            remote_apps_info.append(
                {
                    "Alias": app.AppAlias,
                    "DisplayName": app.DisplayName,
                }
            )

        return remote_apps_info

    def get_collection_info(self, collection_name: str) -> dict:
        """Getting collection info by collection name

        Args:
            collection_name (str): contains collection name

        Returns:
            result (dict): json response
        """
        collections = (
            self._wmi_termserv_handler.Win32_RDCentralPublishedDeploymentSettings(
                PublishingFarm=collection_name
            )
        )
        # Сheck that the collections are not empty and they have one element
        if collections and len(collections) == 1:
            [collection] = collections
            return {"gateway_name": collection.GatewayName}
        return {"gateway_name": None}
